@@ -29,7 +29,8 @@
 #include "logging.h"
 
 // mir
-#include <mir/shell/session_manager.h>
+#include <mir/shell/session.h>
+#include <mir/shell/focus_controller.h>
 #include <mir/shell/surface.h>
 
 // Qt
@@ -53,7 +54,7 @@ ApplicationManager::ApplicationManager(QObject *parent)
 :   ApplicationManagerInterface(parent)
 ,   m_focusedApplication(nullptr)
 ,   m_applicationToBeFocused(nullptr)
-,   m_lifecycleExceptions(QStringList() << "music-app")
+,   m_lifecycleExceptions(QStringList() << "com.ubuntu.music")
 ,   m_taskController(TaskController::singleton())
 ,   m_fenceNext(false)
 {
@@ -172,7 +173,7 @@ bool ApplicationManager::focusApplication(const QString &appId)
         int from = m_applications.indexOf(application);
         move(from, m_applications.length()-1);
     } else {
-        m_mirServer->the_session_manager()->set_focus_to(application->session());
+        m_mirServer->the_focus_controller()->set_focus_to(application->session());
     }
 
     // FIXME(dandrader): lying here. The operation is async. So we will only know whether
@@ -186,7 +187,7 @@ void ApplicationManager::unfocusCurrentApplication()
     DLOG("ApplicationManager::unfocusCurrentApplication (this=%p)", this);
 
     m_applicationToBeFocused = nullptr;
-    m_mirServer->the_session_manager()->set_focus_to(NULL); //FIXME(greyback)
+    m_mirServer->the_focus_controller()->set_focus_to(NULL); //FIXME(greyback)
 }
 
 Application* ApplicationManager::startApplication(const QString &appId,
@@ -238,6 +239,7 @@ void ApplicationManager::onProcessStartReportReceived(const QString &appId, cons
             return;
         }
         add(application);
+        Q_EMIT focusRequested(appId);
     }
 }
 
@@ -295,6 +297,7 @@ void ApplicationManager::onProcessStopped(const QString &appId, const bool unexp
             removeApplication = true;
         } else if (application->state() == Application::Suspended) {
             application->setState(Application::Stopped);
+            application->setSession(nullptr);
         }
 
         if (removeApplication) {
@@ -314,7 +317,7 @@ void ApplicationManager::onFocusRequested(const QString& appId)
 {
     DLOG("ApplicationManager::onFocusRequested (this=%p, appId=%s)", this, qPrintable(appId));
 
-    focusApplication(appId);
+    Q_EMIT focusRequested(appId);
 }
 
 void ApplicationManager::onResumeRequested(const QString& appId)
@@ -367,7 +370,8 @@ void ApplicationManager::authorizeSession(const quint64 pid, bool &authorized)
     QByteArray command = cmdline.readLine().replace('\0', ' ');
 
     // FIXME: special exception for the OSK - maliit-server - not very secure
-    if (command.startsWith("maliit-server") || command.startsWith("/usr/lib/arm-linux-gnueabihf/qt5/libexec/QtWebProcess")) {
+    if (command.startsWith("maliit-server") || command.startsWith("/usr/lib/arm-linux-gnueabihf/qt5/libexec/QtWebProcess")
+        || command.startsWith("/usr/bin/signon-ui")) {
         authorized = true;
         m_fenceNext = true;
         return;
@@ -385,7 +389,15 @@ void ApplicationManager::authorizeSession(const quint64 pid, bool &authorized)
     QString desktopFileName = regExpMatch.captured(1);
     DLOG("Process supplied desktop_file_hint, loading '%s'", desktopFileName.toLatin1().data());
 
-    DesktopFileReader* desktopData = new DesktopFileReader(desktopFileName);
+    // FIXME: right now we support --desktop_file_hint=appId for historical reasons. So let's try that in
+    // case we didn't get an existing .desktop file path
+    DesktopFileReader* desktopData;
+    if (QFileInfo(desktopFileName).exists()) {
+        desktopData = new DesktopFileReader(QFileInfo(desktopFileName));
+    } else {
+        desktopData = new DesktopFileReader(desktopFileName);
+    }
+
     if (!desktopData->loaded()) {
         delete desktopData;
         LOG("ApplicationManager REJECTED connection from app with pid %lld as desktop_file_hint file not found", pid);
@@ -426,7 +438,7 @@ void ApplicationManager::authorizeSession(const quint64 pid, bool &authorized)
     authorized = true;
 }
 
-void ApplicationManager::onSessionStarting(std::shared_ptr<msh::ApplicationSession> const& session)
+void ApplicationManager::onSessionStarting(std::shared_ptr<msh::Session> const& session)
 {
     DLOG("ApplicationManager::onSessionStarting (this=%p, application=%s)", this, session->name().c_str());
 
@@ -446,25 +458,28 @@ void ApplicationManager::onSessionStarting(std::shared_ptr<msh::ApplicationSessi
     }
 }
 
-void ApplicationManager::onSessionStopping(std::shared_ptr<msh::ApplicationSession> const& session)
+void ApplicationManager::onSessionStopping(std::shared_ptr<msh::Session> const& session)
 {
     DLOG("ApplicationManager::onSessionStopping (this=%p, application=%s)", this, session->name().c_str());
 
     // in case application closed not by hand of shell, check again here:
     Application* application = findApplicationWithSession(session);
     if (application) {
+        application->setState(Application::Stopped);
+        application->setSession(nullptr);
+        m_dbusWindowStack->WindowDestroyed(0, application->appId());
+
         if (application == m_focusedApplication) {
             // TODO(greyback) What to do?? Focus next app, or unfocus everything??
             m_focusedApplication = NULL;
             remove(application);
+            delete application;
             Q_EMIT focusedApplicationIdChanged();
         }
-        application->setState(Application::Stopped);
-        m_dbusWindowStack->WindowDestroyed(0, application->appId());
     }
 }
 
-void ApplicationManager::onSessionFocused(std::shared_ptr<msh::ApplicationSession> const& session)
+void ApplicationManager::onSessionFocused(std::shared_ptr<msh::Session> const& session)
 {
     DLOG("ApplicationManager::onSessionFocused (this=%p, application=%s)", this, session->name().c_str());
     Application* application = findApplicationWithSession(session);
@@ -492,7 +507,7 @@ void ApplicationManager::onSessionUnfocused()
         Q_ASSERT(m_focusedApplication->focused());
         m_focusedApplication->setFocused(false);
 
-        if (!m_lifecycleExceptions.contains(m_focusedApplication->appId())
+        if (m_lifecycleExceptions.filter(m_focusedApplication->appId().section('_',0,0)).empty()
                 && m_focusedApplication->state() != Application::Suspended) {
             m_focusedApplication->setState(Application::Suspended);
         }
@@ -506,7 +521,7 @@ void ApplicationManager::onSessionUnfocused()
     }
 }
 
-void ApplicationManager::onSessionCreatedSurface(msh::ApplicationSession const* session,
+void ApplicationManager::onSessionCreatedSurface(msh::Session const* session,
                                                std::shared_ptr<msh::Surface> const& surface)
 {
     DLOG("ApplicationManager::onSessionCreatedSurface (this=%p)", this);
@@ -524,11 +539,13 @@ void ApplicationManager::onSessionCreatedSurface(msh::ApplicationSession const*
 
 void ApplicationManager::setFocused(Application *application)
 {
+    DLOG("ApplicationManager::setFocused (appId=%s)", qPrintable(application->appId()));
+
     if (application == m_focusedApplication)
         return;
 
     // set state of previously focused app to suspended
-    if (m_focusedApplication && !m_lifecycleExceptions.contains(m_focusedApplication->appId())) {
+    if (m_focusedApplication && m_lifecycleExceptions.filter(m_focusedApplication->appId().section('_',0,0)).empty()) {
         m_focusedApplication->setState(Application::Suspended);
     }
 
