@@ -23,17 +23,38 @@
 
 #include "application.h"
 
-class ApplicationManager;
+// upstart
+extern "C" {
+    #include "upstart-app-launch.h"
+}
+
 class TaskController : public QObject
 {
     Q_OBJECT
 public:
-    explicit TaskController(ApplicationManager* parent);
+    static TaskController* singleton();
     ~TaskController();
 
-    void do_resume(Application* application);
-    void do_suspend(Application* application);
-    void do_respawn(Application* application);
+    bool start(const QString& appId, const QStringList& args);
+    bool stop(const QString& appId);
+
+    bool suspend(const QString& appId);
+    bool resume(const QString& appId);
+
+    bool appIdHasProcessId(const QString& appId, const quint64 pid);
+
+Q_SIGNALS:
+    void processStartReport(const QString& appId, const bool failure);
+    void processStopped(const QString& appId, const bool unexpectedly);
+    void requestFocus(const QString& appId);
+    void requestResume(const QString& appId);
+
+private:
+    TaskController(QObject *parent = 0);
+
+    static TaskController* m_theTaskController;
+    upstart_app_launch_app_observer_t preStartCallback, startedCallback, stopCallback, focusCallback, resumeCallback;
+    upstart_app_launch_app_failed_observer_t failureCallback;
 };
 
 #endif // TASKCONTROLLER_H
