@@ -17,10 +17,9 @@
 #include "sessionauthorizer.h"
 #include "logging.h"
 
-SessionAuthorizer::SessionAuthorizer(std::shared_ptr<mir::shell::SessionManager> const& sessionManager, QObject *parent)
+SessionAuthorizer::SessionAuthorizer(QObject *parent)
     :   QObject(parent)
 {
-    Q_UNUSED(sessionManager)
 }
 
 SessionAuthorizer::~SessionAuthorizer()
