@@ -22,7 +22,6 @@
 
 // mir
 #include <mir/frontend/session_authorizer.h>
-#include <mir/shell/session_manager.h>
 
 // Qt
 #include <QObject>
@@ -32,7 +31,7 @@ class SessionAuthorizer : public QObject, public mir::frontend::SessionAuthorize
     Q_OBJECT
 
 public:
-    SessionAuthorizer(std::shared_ptr<mir::shell::SessionManager> const& sessionManager, QObject *parent = 0);
+    SessionAuthorizer(QObject *parent = 0);
     ~SessionAuthorizer();
 
     virtual bool connection_is_allowed(pid_t pid) override;
